﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace GCSOLAR
{
    public partial class FormEnterWaveLengths : Form
    {
        private bool disableWater = true;

        public FormEnterWaveLengths()
        {
            InitializeComponent();
            this.ActiveControl = lblHeader;
            rbtnNo.Checked = true;
            lblWaterType.Visible = true;
            txtWaterBody.Visible = true;

            int jmin = Common.minwav;
            int jmax = Common.maxwav;
            double[] wave = Common.getWave();
            // Set watermarks for each textbox
            txtMinWL.ForeColor = Color.Black;
            txtMinWL.Text = wave[jmin -1].ToString();
            this.txtMinWL.Leave += new System.EventHandler(this.txtMinWL_Leave);
            this.txtMinWL.Enter += new System.EventHandler(this.txtMinWL_Enter);

            txtMaxWL.ForeColor = Color.Black;
            txtMaxWL.Text = wave[jmax - 1].ToString();
            this.txtMaxWL.Leave += new System.EventHandler(this.txtMaxWL_Leave);
            this.txtMaxWL.Enter += new System.EventHandler(this.txtMaxWL_Enter);

            txtWaterBody.ForeColor = Color.Black;
            txtWaterBody.Text = Common.bodyWaterName;
            this.txtWaterBody.Leave += new System.EventHandler(this.txtWaterBody_Leave);
            this.txtWaterBody.Enter += new System.EventHandler(this.txtWaterBody_Enter);

        }

        /// <summary>
        /// This event sets the text in the textbox to a stored value in the program whenever
        /// the user leaves the textbox (user places the cursor somewhere else) without 
        /// entering anything.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void txtMinWL_Leave(object sender, EventArgs e)
        {
            if (txtMinWL.Text == "")
            {
                int j = Common.minwav;
                double[] wave = Common.getWave();
                txtMinWL.Text = wave[j -1].ToString();
                txtMinWL.ForeColor = Color.Black;
            }
        }

        /// <summary>
        /// This event removes the text in the textbox whenever the user places the cursor in it.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void txtMinWL_Enter(object sender, EventArgs e)
        {
            int j = Common.minwav;
            double[] wave = Common.getWave();
            if (txtMinWL.Text == wave[j - 1].ToString())
            {
                txtMinWL.Text = "";
                txtMinWL.ForeColor = Color.Black;
            }
        }

        /// <summary>
        /// This event sets the text in the textbox to a stored value in the program whenever
        /// the user leaves the textbox (user places the cursor somewhere else) without 
        /// entering anything.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void txtMaxWL_Leave(object sender, EventArgs e)
        {
            if (txtMaxWL.Text == "")
            {
                int j = Common.maxwav;
                double[] wave = Common.getWave();
                txtMaxWL.Text = wave[j - 1].ToString();
                txtMaxWL.ForeColor = Color.Black;
            }
        }

        /// <summary>
        /// This event removes the text in the textbox whenever the user places the cursor in it.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void txtMaxWL_Enter(object sender, EventArgs e)
        {
            int j = Common.maxwav;
            double[] wave = Common.getWave();
            if (txtMaxWL.Text == wave[j - 1].ToString())
            {
                txtMaxWL.Text = "";
                txtMaxWL.ForeColor = Color.Black;
            }
        }

        /// <summary>
        /// This event sets the text in the textbox to a stored value in the program whenever
        /// the user leaves the textbox (user places the cursor somewhere else) without 
        /// entering anything.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void txtWaterBody_Leave(object sender, EventArgs e)
        {
            if (txtWaterBody.Text.Trim() == "")
            {
                txtWaterBody.Text = Common.bodyWaterName;
                txtWaterBody.ForeColor = Color.Black;
            }
        }

        /// <summary>
        /// This event removes the text in the textbox whenever the user places the cursor in it.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void txtWaterBody_Enter(object sender, EventArgs e)
        {
            if (txtWaterBody.Text == Common.bodyWaterName)
            {
                txtWaterBody.Text = "";
                txtWaterBody.ForeColor = Color.Black;
            }
        }

        /// <summary>
        /// This method handles the entries of minimum and maximum wavelengths and opens another 
        /// Windows for the entry of absorptions.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnEnterAbs_Click(object sender, EventArgs e)
        {
            string errorMessage;

            if (!ValidMinWave(txtMinWL.Text, out errorMessage))
            {
                errorProvider1.SetError(txtMinWL, errorMessage);
                return;
            }

            if (!ValidMaxWave(txtMaxWL.Text, out errorMessage))
            {
                errorProvider1.SetError(txtMaxWL, errorMessage);
                return;
            }

            if (rbtnNo.Checked)
            {
                if (!ValidWaterBodyName(txtWaterBody.Text, out errorMessage))
                {
                    errorProvider1.SetError(txtWaterBody, errorMessage);
                    return;
                }
            }

            double minimumWave = Convert.ToDouble(txtMinWL.Text.Trim());
            double maximumWave = Convert.ToDouble(txtMaxWL.Text.Trim());

            if (minimumWave >= maximumWave)
            {
                errorMessage = "Error: Minimum wavelength must be less than maximum wavelength.";
                errorProvider1.SetError(txtMinWL, errorMessage);
                errorMessage = "Error: Maximum wavelength must be greater than maximum wavelength.";
                errorProvider1.SetError(txtMaxWL, errorMessage);
                return;
            }

            double[] wave = Common.getWave();

            if (minimumWave != 0)
            {
                double wave1 = minimumWave;
                for (int j = 1; j <= 46; j++)
                {
                    if (Math.Abs(wave1 - wave[j - 1]) < 0.001)
                    {
                        Common.minwav = j;
                        break;
                    }
                }
            }

            if (maximumWave != 0)
            {
                double wave2 = maximumWave;
                for (int j = 1; j <= 46; j++)
                {
                    if (Math.Abs(wave2 - wave[j - 1]) < 0.001)
                    {
                        Common.maxwav = j;
                        break;
                    }
                }
            }

            if (rbtnNo.Checked == true)
            {
                Common.bodyWaterName = txtWaterBody.Text.Trim();
                Common.bodyWaterType = "Natural Water";
            }
            if (rbtnYes.Checked == true)
            {
                Common.bodyWaterName = "Pure Water";
                Common.bodyWaterType = "Pure Water";
            }
            FormAbsorptions formAbsorptions = new FormAbsorptions();
            formAbsorptions.setVisibleWaterColumn(disableWater);
            formAbsorptions.ShowDialog();
            this.Close();
        }

        private bool ValidWaterBodyName(string txtName, out string errorMessage)
        {
            if (string.IsNullOrEmpty(txtName))
            {
                errorMessage = "Name of water body is required.";
                return false;
            }

            errorMessage = "";
            return true;
        }

        private void txtWaterBody_Validated(object sender, EventArgs e)
        {
            // If all conditions have been met, clear the ErrorProvider of errors.
            errorProvider1.SetError(txtWaterBody, "");
        }

        private void txtWaterBody_Validating(object sender, CancelEventArgs e)
        {
            string errorMessage;
            if (!ValidWaterBodyName(txtWaterBody.Text, out errorMessage))
            {
                // Cancel the event and select the text to be corrected by the user.
                e.Cancel = true;

                // Set the ErrorProvider error with the text to display. 
                this.errorProvider1.SetError(txtWaterBody, errorMessage);
            }
        }

        /// <summary>
        /// This method validates the minimum wavelength.
        /// </summary>
        /// <param name="txtMinwave"></param>
        /// <param name="errorMessage"></param>
        /// <returns></returns>
        private bool ValidMinWave(string txtMinwave, out string errorMessage)
        {
            double minwave;

            if (string.IsNullOrEmpty(txtMinwave))
            {
                errorMessage = "Minimum wavelength is required.";
                return false;
            }

            if (txtMinwave.Contains(" "))
            {
                errorMessage = "Do not enter spaces.";
                return false;
            }

            if (!Double.TryParse(txtMinwave, out minwave))
            {
                errorMessage = "Minimum wavelength is not double.";
                return false;
            }

            if (minwave < 280.0 || minwave > 800.0)
            {
                errorMessage = "Minimum wavelength has to be a number between 280nm and 800nm";
                return false;
            }

            bool valueFound = false;
            double[] wave = Common.getWave();
            for (int i = 0; i < wave.Length; i++)
            {
                if (Math.Abs(minwave - wave[i]) < 0.001) valueFound = true;
            }
            if (!valueFound)
            {
                errorMessage = "Minimum wavelength must be one of the allowed values shown below.";
                return false;
            }

            errorMessage = "";
            return true;
        }

        /// <summary>
        /// This method validates the maximum wavelength.
        /// </summary>
        /// <param name="txtMaxwave"></param>
        /// <param name="errorMessage"></param>
        /// <returns></returns>
        private bool ValidMaxWave(string txtMaxwave, out string errorMessage)
        {
            double maxwave;

            if (string.IsNullOrEmpty(txtMaxwave))
            {
                errorMessage = "Maximum wavelength is required.";
                return false;
            }

            if (txtMaxwave.Contains(" "))
            {
                errorMessage = "Do not enter spaces.";
                return false;
            }

            if (!Double.TryParse(txtMaxwave, out maxwave))
            {
                errorMessage = "Maximum wavelength is not double.";
                return false;
            }

            if (maxwave < 280.0 || maxwave > 800.0)
            {
                errorMessage = "Maximum wavelength has to be an integer between 280nm and 800nm";
                return false;
            }

            bool valueFound = false;
            double[] wave = Common.getWave();
            for (int i = 0; i < wave.Length; i++)
            {
                if (Math.Abs(maxwave - wave[i]) < 0.001) valueFound = true;
            }
            if (!valueFound)
            {
                errorMessage = "Maximum wavelength must be one of the allowed values shown below.";
                return false;
            }

            errorMessage = "";
            return true;
        }

        private void txtMinWL_Validating(object sender, CancelEventArgs e)
        {
            string errorMessage;
            if (!ValidMinWave(txtMinWL.Text, out errorMessage))
            {
                // Cancel the event and select the text to be corrected by the user.
                e.Cancel = true;

                // Set the ErrorProvider error with the text to display. 
                this.errorProvider1.SetError(txtMinWL, errorMessage);
            }
        }

        private void txtMinWL_Validated(object sender, EventArgs e)
        {
            // If all conditions have been met, clear the ErrorProvider of errors.
            errorProvider1.SetError(txtMinWL, "");
        }

        private void txtMaxWL_Validating(object sender, CancelEventArgs e)
        {
            string errorMessage;
            if (!ValidMaxWave(txtMaxWL.Text, out errorMessage))
            {
                // Cancel the event and select the text to be corrected by the user.
                e.Cancel = true;

                // Set the ErrorProvider error with the text to display. 
                this.errorProvider1.SetError(txtMaxWL, errorMessage);
            }
        }

        private void txtMaxWL_Validated(object sender, EventArgs e)
        {
            // Cancel the event and select the text to be corrected by the user.
            errorProvider1.SetError(txtMaxWL, "");
        }

        private void rbtnYes_CheckedChanged(object sender, EventArgs e)
        {
            lblWaterType.Visible = false;
            txtWaterBody.Visible = false;
            btnSave.Visible = false;
            disableWater = true;
        }

        private void rbtnNo_CheckedChanged(object sender, EventArgs e)
        {
            lblWaterType.Visible = true;
            txtWaterBody.Visible = true;
            btnSave.Visible = true;
            disableWater = false;
        }

        private void FormEnterWaveLengths_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = false;
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            if (rbtnNo.Checked == true)
            {
                Common.bodyWaterName = txtWaterBody.Text.Trim();
                Common.bodyWaterType = "Natural Water";
            }
            if (rbtnYes.Checked == true)
            {
                Common.bodyWaterName = "Pure Water";
                Common.bodyWaterType = "Pure Water";
            }
            MessageBox.Show("Name of water body was saved.");
        }

    }
}
