﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;


namespace GCSOLAR
{
    public partial class FormMultiplePlotsTod : Form
    {
        private DataTable[] dt;
        private String[] var;
        private string xAxis;
        private string yAxis;
        private string strLegend;
        private string strUnits;
        private string param;
        private string season;

        public FormMultiplePlotsTod()
        {
            InitializeComponent();
        }

        // Assign table holding the data, and X and Y axes labels to their corresponding 
        // private members of this class.
        public void setTable(DataTable[] dt1, String[] depth, string strLegend, string units, string parameter, string season, string strX, string strY)
        {
            int num = dt1.Length;
            DataTable[] dt2 = new DataTable[num];
            for (int i = 0; i < num; i++)
            {
                dt2[i] = dt1[i].DefaultView.ToTable(true);
            }
            this.dt = dt2;
            this.var = depth;
            this.strLegend = strLegend;
            this.strUnits = units;
            this.param = parameter;
            this.season = season;
            this.xAxis = strX;
            this.yAxis = strY;
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            // Create an instance of the save file dialog box.
            SaveFileDialog saveFileDialog1 = new SaveFileDialog();

            // Set filter options and filter index.
            saveFileDialog1.Filter = "PNG Files (.png)|*.png|All Files (*.*)|*.*";
            saveFileDialog1.FilterIndex = 1;

            saveFileDialog1.RestoreDirectory = true;

            // Call the ShowDialog method to show the dialog box.
            System.Windows.Forms.DialogResult userClickedOK = saveFileDialog1.ShowDialog();

            if (userClickedOK == System.Windows.Forms.DialogResult.OK)
            {
                this.chart1.SaveImage(saveFileDialog1.FileName, ChartImageFormat.Png);
            }              
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void FormMultiplePlotsTod_Load(object sender, EventArgs e)
        {
            addDataChart();
        }

        /// <summary>
        /// This method plots the results of time-of-day simulations.  Rates are plotted as
        /// a function of hours. It uses the Chart control of C#.
        /// </summary>
        private void addDataChart()
        {

            Legend legend = new Legend("legend");
            LegendItem[] legItem = new LegendItem[this.var.Length];

            Random random1 = new Random();
            for (int i = 0; i < this.var.Length; i++)
            {
                // Add series
                chart1.Series.Add(this.var[i]);
                
                // Set the type of curve.
                chart1.Series[this.var[i]].ChartType = SeriesChartType.Spline;

                // Bind the table holding the data to the Chart control.
                chart1.Series[this.var[i]].Points.DataBindXY(dt[i].DefaultView, xAxis, dt[i].DefaultView, yAxis);  

                // Set the legend's text.
                string leg = strLegend + this.var[i] + strUnits;
                
                // Set the color of the legend.
                Color c = Color.FromArgb(random1.Next(0, 255), random1.Next(0, 255), random1.Next(0, 255));
                legItem[i] = new LegendItem(leg, c, "");               
                legend.CustomItems.Add(legItem[i]);

                // Set the color of the curve.
                chart1.Series[this.var[i]].Color = c;                         
            }

            legend.Font = new Font("Arial", 14, FontStyle.Bold);
            chart1.Legends.Add(legend);


            // Specify the X and Y axes data.
            chart1.Series[this.var[0]].XValueMember = xAxis;
            chart1.Series[this.var[0]].YValueMembers = yAxis;

            // Set the graph's title.
            string strTitle = "";
            string strXaxis = "";
            if (xAxis.Equals("Time of Day (hours)"))
            {
                if (strLegend.Equals("Depth = "))
                {
                    if (!season.Equals(""))
                    {
                        strTitle = Common.contaminantName + " Photolysis Rate Constants vs Time-of-Day; " + " Latitude = " + param + ", Season = " + season;
                    }
                    else
                    {
                        strTitle = Common.contaminantName + " Photolysis Rate Constants vs Time-of-Day; " + " Latitude = " + param;
                    }
                }

                if (strLegend.Equals("Latitude = "))
                {
                    if (!season.Equals(""))
                    {
                        strTitle = Common.contaminantName + " Photolysis Rate Constants vs Time-of-Day; " + " Depth = " + param + " cm" + ", Season = " + season;
                    }
                    else
                    {
                        strTitle = Common.contaminantName + " Photolysis Rate Constants vs Time-of-Day; " + " Depth = " + param + " cm";
                    }
                }

                //strTitle = Common.contaminantName + " Photolysis Rate Constants vs Time-of-Day; ";
                strXaxis = "Time of day";
                chart1.ChartAreas[0].AxisX.Minimum = 0.0;
                chart1.ChartAreas[0].AxisX.Maximum = 24.0;
                chart1.ChartAreas[0].AxisX.Interval = 2.0;
            }
            else
            {
                if (strLegend.Equals("Depth = "))
                {
                    if (!season.Equals(""))
                    {
                        strTitle = Common.contaminantName + " Photolysis Rate Constants vs Solar Altitude; " + " Latitude = " + param + ", Season = " + season;
                    }
                    else
                    {
                        strTitle = Common.contaminantName + " Photolysis Rate Constants vs Time-of-Day; " + " Latitude = " + param;
                    }
                }

                if (strLegend.Equals("Latitude = "))
                {
                    if (!season.Equals(""))
                    {
                        strTitle = Common.contaminantName + " Photolysis Rate Constants vs Solar Altitude; " + " Depth = " + param + " cm" + ", Season = " + season;
                    }
                    else
                    {
                        strTitle = Common.contaminantName + " Photolysis Rate Constants vs Time-of-Day; " + " Depth = " + param + " cm";
                    }
                }
                //strTitle = Common.contaminantName + " Photolysis Rate Constants vs Solar Altitude";
                strXaxis = "Solar Altitude (degrees)";
                chart1.ChartAreas[0].AxisX.Minimum = 0.0;
                chart1.ChartAreas[0].AxisX.Maximum = 90.0;
                chart1.ChartAreas[0].AxisX.Interval = 10.0;
            }
            chart1.Titles.Add(strTitle);

            // Specify the title's font.
            chart1.Titles[0].Font = new Font("Arial", 12, FontStyle.Bold);

            // Specify the X and Y axes's titles.
            chart1.ChartAreas[0].AxisX.Title = strXaxis;
            chart1.ChartAreas[0].AxisY.Title = "Photolysis Rate Constants (s\x207B\x00B9)";

            // Set the fonts of the X and Y axes's titles.
            chart1.ChartAreas[0].AxisX.TitleFont = new Font("Arial", 12, FontStyle.Bold);
            chart1.ChartAreas[0].AxisY.TitleFont = new Font("Arial", 12, FontStyle.Bold);

            Random random = new Random();
            for (int i = 0; i < this.var.Length; i++)
            {
                // Set the width (thickness) of the curve.
                chart1.Series[this.var[i]].BorderWidth = 2;

                // Set the font of the curve.
                chart1.Series[this.var[i]].Font = new Font("Arial", 14, FontStyle.Bold);
            }
            
        }
    }
}
