﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;

namespace GCSOLAR
{
    public partial class FormPlotAbsorptions : Form
    {
        private DataTable dt = new DataTable();
        private string xAxis;
        private string yAxis;

        public FormPlotAbsorptions()
        {
            InitializeComponent();
        }

        // Assign table holding the data, and X and Y axes labels to their corresponding 
        // private members of this class.
        public void setTable(DataTable dt1, string strX, string strY)
        {
            this.dt = dt1;
            this.xAxis = strX;
            this.yAxis = strY;
        }

        /// <summary>
        /// This method closes this Windows form.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// This is the load method of this form. It invokes the addDataChart method (see below).  
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void FormPlotAbsorptions_Load(object sender, EventArgs e)
        {
            addDataChart();
        }

        /// <summary>
        /// This method handles the plotting of the data by using the Chart control.  It adds
        /// the data to the Chart control, sets the X and Y axes, labels, title, fonts, and colors.
        /// </summary>
        private void addDataChart()
        {
            // Attach the table holding the data to the Chart control.
            chart1.DataSource = dt.DefaultView;

            // Specify the X and Y axes data.
            chart1.Series["Absorptions vs Wavelengths"].XValueMember = xAxis;
            chart1.Series["Absorptions vs Wavelengths"].YValueMembers = yAxis;

            // Set the graph's title.
            string strTitle = "";
            if (yAxis.Contains("Chemical"))
            {
                strTitle = "Name of water body: " + Common.bodyWaterName + ",  Contaminant: " + Common.contaminantName;               
            }
            else
            {
                strTitle = "Name of water body: " + Common.bodyWaterName;
            }
            chart1.Titles.Add(strTitle);

            // Specify the title's font.
            chart1.Titles[0].Font = new Font("Arial", 12, FontStyle.Bold);  

            // Specify the X and Y axes's titles.
            chart1.ChartAreas[0].AxisX.Title = xAxis;
            chart1.ChartAreas[0].AxisY.Title = yAxis;

            // Set the fonts of the X and Y axes's titles.
            chart1.ChartAreas[0].AxisX.TitleFont = new Font("Arial", 12, FontStyle.Bold);
            chart1.ChartAreas[0].AxisY.TitleFont = new Font("Arial", 12, FontStyle.Bold);

            // Set the width (thickness) of the curve.
            chart1.Series["Absorptions vs Wavelengths"].BorderWidth = 2;

            // Set the color of the curve.
            chart1.Series["Absorptions vs Wavelengths"].Color = Color.Black;
        }

        /// <summary>
        /// This method saves the plot to a file.  The format of the file is PNG.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSave_Click(object sender, EventArgs e)
        {
            // Create an instance of the save file dialog box.
            SaveFileDialog saveFileDialog1 = new SaveFileDialog();

            // Set filter options and filter index.
            saveFileDialog1.Filter = "PNG Files (.png)|*.png|All Files (*.*)|*.*";
            saveFileDialog1.FilterIndex = 1;

            saveFileDialog1.RestoreDirectory = true;

            // Call the ShowDialog method to show the dialog box.
            System.Windows.Forms.DialogResult userClickedOK = saveFileDialog1.ShowDialog();

            if (userClickedOK == System.Windows.Forms.DialogResult.OK)
            {
                this.chart1.SaveImage(saveFileDialog1.FileName, ChartImageFormat.Png);
            }              
        }

        private void FormPlotAbsorptions_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = false;
        }
    }
}
