﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;

namespace GCSOLAR
{
    public partial class FormPlotKlambdas : Form
    {
        private DataTable dt = new DataTable();
        private string latitude;
        private string depth;
        private string season;
        private string solar;

        public FormPlotKlambdas()
        {
            InitializeComponent();
        }

        public void setTable(DataTable dt1, string strLat, string strDepth, string strSeason, string strSolar)
        {
            this.dt = dt1;
            this.latitude = strLat;
            this.depth = strDepth;
            this.season = strSeason;
            this.solar = strSolar;
        }

        /// <summary>
        /// This method closes the current Windows form.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// This is the load method of this form. It invokes the addDataChart method (see below).  
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void FormPlotKlambdas_Load(object sender, EventArgs e)
        {
            addDataChart();
        }

        /// <summary>
        /// This method plots the Ka-lambdas as a function of wavelengths.  It uses the 
        /// Chart control of C#.
        /// </summary>
        private void addDataChart()
        {
            // Attach the table holding the data to the Chart control.
            chart1.DataSource = dt.DefaultView;

            // Specify the X and Y axes data.
            chart1.Series["Ka Lambdas"].XValueMember = "Wavelength (nm)";
            chart1.Series["Ka Lambdas"].YValueMembers = "Ka-Lambda (sec^(-1) nm^(-1))";

            // Set the graph's title.
            string strTitle = "";
            if (!season.Equals(""))
            {
                strTitle = "K\x2090(\u03BB) vs Wavelengths; " + "Latitude = " + latitude + ", Depth = " + depth + 
                    ", Season = " + season + ", Solar Altitude = " + solar;
            }
            else
            {
                strTitle = "K\x2090(\u03BB) vs Wavelengths; " + "Latitude = " + latitude + ", Depth = " + depth + 
                    ", Solar Altitude = " + solar;
            }
            chart1.Titles.Add(strTitle);

            // Specify the title's font.
            chart1.Titles[0].Font = new Font("Arial", 12, FontStyle.Bold);

            // Specify the X and Y axes's titles.
            chart1.ChartAreas[0].AxisX.Title = "\u03BB (nm)";
            chart1.ChartAreas[0].AxisY.Title = "K\x2090(\u03BB) (s\x207B\x00B9 nm\x207B\x00B9)";

            // Set the fonts of the X and Y axes's titles.
            chart1.ChartAreas[0].AxisX.TitleFont = new Font("Arial", 12, FontStyle.Bold);
            chart1.ChartAreas[0].AxisY.TitleFont = new Font("Arial", 12, FontStyle.Bold);

            // Set the numeric format of the X axis.
            chart1.ChartAreas[0].AxisX.LabelStyle.Format = "###.##";

            // Set the width (thickness) of the curve.
            chart1.Series["Ka Lambdas"].BorderWidth = 2;

            // Set the color of the curve.
            chart1.Series["Ka Lambdas"].Color = Color.Black;
        }

        /// <summary>
        /// This method saves the plot to a file.  The format of the file is PNG.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSave_Click(object sender, EventArgs e)
        {
            // Create an instance of the save file dialog box.
            SaveFileDialog saveFileDialog1 = new SaveFileDialog();

            // Set filter options and filter index.
            saveFileDialog1.Filter = "PNG Files (.png)|*.png|All Files (*.*)|*.*";
            saveFileDialog1.FilterIndex = 1;

            saveFileDialog1.RestoreDirectory = true;

            // Call the ShowDialog method to show the dialog box.
            System.Windows.Forms.DialogResult userClickedOK = saveFileDialog1.ShowDialog();
            
            if (userClickedOK == System.Windows.Forms.DialogResult.OK)
            {
                this.chart1.SaveImage(saveFileDialog1.FileName, ChartImageFormat.Png);
            }              
        }

        private void FormPlotKlambdas_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = false;
        }
     
    }
}
