﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;

namespace GCSOLAR
{
    public partial class FormPlotTimeOfDay : Form
    {
        private DataTable dt = new DataTable();
        private string xAxis;
        private string yAxis;
        private string latitude;
        private string depth;
        private string season;

        /// <summary>
        /// Constructor.
        /// </summary>
        public FormPlotTimeOfDay()
        {
            InitializeComponent();
        }

        // Assign table holding the data, and X and Y axes labels to their corresponding 
        // private members of this class.
        public void setTable(DataTable dt1, string strLat, string strDepth, string strSeason, string strX, string strY)
        {
            DataTable dt2 = dt1.DefaultView.ToTable(true);
            this.dt = dt2;
            this.latitude = strLat;
            this.depth = strDepth;
            this.season = strSeason;
            this.xAxis = strX;
            this.yAxis = strY;
        }

        /// <summary>
        /// This is the load method of this form. It invokes the addDataChart method (see below).  
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void FormPlotTimeOfDay_Load(object sender, EventArgs e)
        {
             addDataChart();
        }

        /// <summary>
        /// This method plots the results of time-of-day simulations.  Rates are plotted as
        /// a function of hours. It uses the Chart control of C#.
        /// </summary>
        private void addDataChart()
        {
            // Attach the table holding the data to the Chart control.
            chart1.DataSource = dt.DefaultView;

            // Specify the X and Y axes data.
            chart1.Series["Rate Constants as a function of time of day"].XValueMember = xAxis;
            chart1.Series["Rate Constants as a function of time of day"].YValueMembers = yAxis;

            // Set the graph's title.
            string strTitle = "";
            string strXaxis = "";
            if (xAxis.Equals("Time of Day (hours)"))
            {
                if (!season.Equals(""))
                {
                    strTitle = Common.contaminantName + " Photolysis Rate Constants vs Time-of-Day; " + "Latitude = " + latitude + 
                        ", Depth = " + depth + ", Season = " + season;
                }
                else
                {
                    strTitle = Common.contaminantName + " Photolysis Rate Constants vs Time-of-Day; " + "Latitude = " + latitude +
                        ", Depth = " + depth;
                }
                strXaxis = "Time of day";
                chart1.ChartAreas[0].AxisX.Minimum = 0.0;
                chart1.ChartAreas[0].AxisX.Maximum = 24.0;
                chart1.ChartAreas[0].AxisX.Interval = 2.0;
            }
            else
            {
                if (!season.Equals(""))
                {
                    strTitle = Common.contaminantName + " Photolysis Rate Constants vs Solar Altitude; " + "Latitude = " + latitude +
                        ", Depth = " + depth + ", Season = " + season;
                }
                else
                {
                    strTitle = Common.contaminantName + " Photolysis Rate Constants vs Solar Altitude; " + "Latitude = " + latitude +
                        ", Depth = " + depth;
                }
                strXaxis = "Solar Altitude (degrees)";
                chart1.ChartAreas[0].AxisX.Minimum = 0.0;
                chart1.ChartAreas[0].AxisX.Maximum = 90.0;
                chart1.ChartAreas[0].AxisX.Interval = 10.0;
            }
            chart1.Titles.Add(strTitle);

            // Specify the title's font.
            chart1.Titles[0].Font = new Font("Arial", 12, FontStyle.Bold);

            // Specify the X and Y axes's titles.
            chart1.ChartAreas[0].AxisX.Title = strXaxis;
            chart1.ChartAreas[0].AxisY.Title = "Photolysis Rate Constants (s\x207B\x00B9)";

            // Set the fonts of the X and Y axes's titles.
            chart1.ChartAreas[0].AxisX.TitleFont = new Font("Arial", 12, FontStyle.Bold);
            chart1.ChartAreas[0].AxisY.TitleFont = new Font("Arial", 12, FontStyle.Bold);

            // Set the width (thickness) of the curve.
            chart1.Series["Rate Constants as a function of time of day"].BorderWidth = 2;

            // Set the color of the curve.
            chart1.Series["Rate Constants as a function of time of day"].Color = Color.Black;           
        }

        /// <summary>
        /// This method closes the current Windows form.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// This method saves the plot to a file.  The format of the file is PNG.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSave_Click(object sender, EventArgs e)
        {
            // Create an instance of the save file dialog box.
            SaveFileDialog saveFileDialog1 = new SaveFileDialog();

            // Set filter options and filter index.
            saveFileDialog1.Filter = "PNG Files (.png)|*.png|All Files (*.*)|*.*";
            saveFileDialog1.FilterIndex = 1;

            saveFileDialog1.RestoreDirectory = true;

            // Call the ShowDialog method to show the dialog box.
            System.Windows.Forms.DialogResult userClickedOK = saveFileDialog1.ShowDialog();

            if (userClickedOK == System.Windows.Forms.DialogResult.OK)
            {
                this.chart1.SaveImage(saveFileDialog1.FileName, ChartImageFormat.Png);
            }              
        }

        private void FormPlotTimeOfDay_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = false;
        }
    }
}
