﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Drawing;
using System.Windows.Forms;
using System.Data;

namespace GCSOLAR
{
   
    public class Latitudes
    {
        private DataGridView gridLat = new DataGridView();

        /// <summary>
        /// This method displays an input box used to enter latitudes.
        /// </summary>
        /// <param name="numLats"></param>
        /// <returns></returns>
        public DialogResult InputLatitudes(int numLats)
        {
            Form form = new Form();
            DataTable table = new DataTable();

            Button buttonOk = new Button();
            Button buttonCancel = new Button();
            Label lblInstructions = new Label();

            form.ClientSize = new Size(600, 600);
            int ycoord = 400;

            buttonOk.SetBounds(300, ycoord, 75, 30);
            buttonCancel.SetBounds(400, ycoord, 75, 30);
            buttonOk.Font = new Font("Arial", 10, FontStyle.Bold);
            buttonCancel.Font = new Font("Arial", 10, FontStyle.Bold);
            lblInstructions.SetBounds(50, 10, 420, 69);
            lblInstructions.Font = new Font("Arial", 10, FontStyle.Bold);
            buttonOk.Anchor = AnchorStyles.Bottom | AnchorStyles.Right;
            buttonCancel.Anchor = AnchorStyles.Bottom | AnchorStyles.Right;
            lblInstructions.Anchor = AnchorStyles.Bottom | AnchorStyles.Right;

            lblInstructions.Text = "Enter the latitudes in the table below and click OK. " +
                                   "Do not enter latitudes >= 70.0.";

            gridLat.Location = new Point(50, 70);
            gridLat.Size = new Size(400, 400);
            gridLat.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.Fill;
            gridLat.AutoSizeRowsMode = DataGridViewAutoSizeRowsMode.AllCells;

            form.Controls.Add(gridLat);

            Common.ilatsw = numLats;
            gridLat.CellValidating += new DataGridViewCellValidatingEventHandler(gridLat_CellValidating);
            gridLat.CellEndEdit += new DataGridViewCellEventHandler(gridLat_CellEndEdit);

            DataRow dr1 = null;
            table.Columns.Add("Latitude index");
            table.Columns.Add("Latitude");

            for (int i = 0; i < numLats; i++)
            {
                dr1 = table.NewRow();
                dr1["Latitude index"] = i + 1;
                if (Common.ilattm[i] >= 0.0)
                {
                    dr1["Latitude"] = Common.ilattm[i];
                }
                table.Rows.Add(dr1);
            }
            gridLat.DataSource = table.DefaultView;
            gridLat.BorderStyle = BorderStyle.Fixed3D;
            gridLat.CellBorderStyle = DataGridViewCellBorderStyle.Raised;
            gridLat.ColumnHeadersBorderStyle = DataGridViewHeaderBorderStyle.Raised;
            gridLat.GridColor = Color.DarkRed;
            gridLat.Columns[0].ReadOnly = true;

            int totalRowHeight = 0;
            foreach (DataGridViewRow row in gridLat.Rows)
            {
                totalRowHeight += row.Height;
            }
            if (totalRowHeight < 400)
            {
                gridLat.Height = totalRowHeight;
            }

            gridLat.AllowUserToAddRows = false;

            form.Text = "Latitudes";
            buttonOk.Text = "OK";
            buttonCancel.Text = "Cancel";
            buttonOk.DialogResult = DialogResult.OK;
            buttonCancel.DialogResult = DialogResult.Cancel;

            form.Controls.AddRange(new Control[] { buttonOk, buttonCancel });
            form.Controls.Add(lblInstructions);
            form.FormBorderStyle = FormBorderStyle.FixedDialog;
            form.StartPosition = FormStartPosition.CenterScreen;
            form.MinimizeBox = false;
            form.MaximizeBox = false;
            form.AcceptButton = buttonOk;
            form.CancelButton = buttonCancel;
            bool retry = false;
            DialogResult dialogResult;
            do
            {
                dialogResult = form.ShowDialog();

                if (dialogResult == DialogResult.OK)
                {
                    int j = 0;
                    foreach (DataRow row in table.Rows)
                    {
                        string str = row["Latitude"].ToString().Trim();
                        if (str.Equals(""))
                        {
                            MessageBox.Show("Some entries in the table are empty. Please enter a latitude for each entry in the table.");
                            retry = true;
                            break;
                        }
                        double value = Convert.ToDouble(row["Latitude"].ToString());
                        Common.ilattm[j] = value;
                        j++;
                    }
                }
                else if (dialogResult == DialogResult.Cancel)
                {
                    retry = false;
                }
            } while (retry);

            return dialogResult;
        }

        private void gridLat_CellValidating(object sender, DataGridViewCellValidatingEventArgs e)
        {
            // Confirm that the cell is not empty.
            if (string.IsNullOrEmpty(e.FormattedValue.ToString()))
            {
                gridLat.Rows[e.RowIndex].ErrorText = "Cell must not be empty";
                e.Cancel = true;
                return;
            }

            double value;
            // Confirm that the cell value is integer.
            if (!Double.TryParse(e.FormattedValue.ToString(), out value))
            {
                gridLat.Rows[e.RowIndex].ErrorText = "Cell value must be a number.";
                e.Cancel = true;
                return;
            }

            if (value < 0.0 || value >= 70.0)
            {
                gridLat.Rows[e.RowIndex].ErrorText = "Cell value must be equal to or greater than 0 and less than 70.";
                e.Cancel = true;
                return;
            }
        }

        private void gridLat_CellEndEdit(object sender, DataGridViewCellEventArgs e)
        {
            // Clear the row error in case the user presses ESC.   
            gridLat.Rows[e.RowIndex].ErrorText = String.Empty;
        }
    }
    
}
