﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace GCSOLAR
{
    public partial class ListForm : Form
    {
        public ListForm()
        {
            InitializeComponent();
        }

        public void listChemicalInformation()
        {
            DataTable dtlist = new DataTable();
            Common.Listing(out dtlist);
            dataGridView1.DataSource = dtlist.DefaultView;
            dataGridView1.ColumnHeadersDefaultCellStyle.Font = new Font("Arial", 10, FontStyle.Bold);

            lblName.Text = "Contaminant Name: " + Common.contaminantName;
            lblWater.Text = "Water Type Name: " + Common.bodyWaterName;
            lblAtmos.Text = "Type of Atmosphere: " + Common.typeAtmos;
            lblLon.Text = "Longitude: " + Convert.ToString(Common.xlon);
            lblElev.Text = "Elevation: " + Convert.ToString(Common.elevation) + " km";
            lblQ.Text = "Quantum Yield: " + Convert.ToString(Common.q);
            lblInitialDepth.Text = "Initial Depth: " + Convert.ToString(Common.dinit) + " cm";
            lblDepthIncrement.Text = "Depth Increment: " + Convert.ToString(Common.dinc) + " cm";
            lblFinalDepth.Text = "Final Depth: " + Convert.ToString(Common.dfinal) + " cm";
            lblRI.Text = "Refractive Index: " + Convert.ToString(Common.musubr);


            if (Common.useDeltaz == true)
            {
                lblDepthPoint.Text = "Depth Point: " + Convert.ToString(Common.deltaz) + " cm";
            }
            else
            {
                lblDepthPoint.Text = "Depth Point: None";
            }

            lblSeasons.Text = "Season(s): ";
            for (int i = 0; i < Common.iseasw; i++)
            {
                if (i == Common.iseasw - 1)
                {
                    lblSeasons.Text = lblSeasons.Text + Common.sease[i];
                }
                else
                {
                    lblSeasons.Text = lblSeasons.Text + Common.sease[i] + ", ";
                }              
            }

            lblLats.Text = "Latitude(s): ";
            for (int i = 0; i < Common.ilatsw; i++)
            {
                if (i == Common.ilatsw - 1)
                {
                    lblLats.Text = lblLats.Text + Convert.ToString(Common.ilattm[i]);
                }
                else
                {
                    lblLats.Text = lblLats.Text + Convert.ToString(Common.ilattm[i]) + ", ";
                }
            }

            cmbAbsorptions.Items.Add("Water Absorption Coefficients");
            cmbAbsorptions.Items.Add("Contaminant Absorption Coefficients");
            cmbAbsorptions.SelectedIndex = 0;
        }

        private void button2_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void btnToD_Click(object sender, EventArgs e)
        {
            DataTable dtday = new DataTable();
            DataTable dtKL = new DataTable();
            Depend dp = new Depend();
            string errorMsg = "";
            dp.CalculatePhotolysisRatesHalfLivesTDay(out dtday, out dtKL, out errorMsg);

            // If an error message was returned, display it and return control to the Windows 
            // form.
            if (errorMsg != "")
            {
                MessageBox.Show(errorMsg);
                return;
            }

            TodForm tod = new TodForm(dtday, dtKL);
            tod.ShowDialog();
        }

        private void btnPlotAbsorptions_Click(object sender, EventArgs e)
        {
            DataTable dt = new DataTable();
            DataTable dtData = new DataTable();
            DataRow dr1 = null;

            // Get selected absorptions.
            string itemAbs = cmbAbsorptions.SelectedItem.ToString();
            string xAxis = Common.wavelengthName;
            string absColumn = "";
            string yAxis = "";
            string contaminant = Common.contaminantType.Trim() + " " + Common.contaminantUnits.Trim();

            if (itemAbs.Equals("Water Absorption Coefficients"))
            {
                absColumn = Common.waterNameUnits;
                yAxis = Common.bodyWaterType + " Absorption Coefficients (m**-1)";
            }
            else
            {
                absColumn = contaminant;
                yAxis = contaminant;
            }

            // Add columns to the table that will hold the data to be plotted.
            dtData.Columns.Add(xAxis);
            dtData.Columns.Add(yAxis);

            // Get the data stored in the program.
            Common.Listing(out dt);

            // Build the dtData table with the data to be plotted.
            foreach (DataRow rows in dt.Rows)
            {
                dr1 = dtData.NewRow();
                dr1[xAxis] = rows[xAxis].ToString().Trim();
                dr1[yAxis] = rows[absColumn].ToString().Trim();
                dtData.Rows.Add(dr1);
            }

            // Plot the data.
            FormPlotAbsorptions abs = new FormPlotAbsorptions();
            abs.setTable(dtData, xAxis, yAxis);
            abs.ShowDialog();
        }

        private void ListForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = false;
        }


    }
}
