﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Globalization;

namespace GCSOLAR
{
    public partial class TodForm : Form
    {
        public TodForm(DataTable dtday, DataTable dtKL)
        {
            InitializeComponent();
            dataGridView1.DataSource = dtday.DefaultView;

            // Change the font of "Summary Items" column to Bold.
            dataGridView1.Columns["Summary Items"].DefaultCellStyle.Font = new Font(dataGridView1.DefaultCellStyle.Font, FontStyle.Bold);

            // Adjust the width of the "Summary Items" column to make it wider.
            dataGridView1.Columns["Summary Items"].FillWeight = 150;

            dataGridView1.Visible = true;
            dataGridView2.DataSource = dtKL.DefaultView;

            radioButton1.Checked = true;

            if (Common.ikalam == 0)  // If Ka-lambdas are not being printed, do not show table and controls.
            {
                dataGridView2.Visible = false;
                lblKDepth.Visible = false;
                lblKLat.Visible = false;
                lblKSeason.Visible = false;
                lblKSolar.Visible = false;
                cmbKDepth.Visible = false;
                cmbKLat.Visible = false;
                cmbKSeason.Visible = false;
                cmbKSolar.Visible = false;
                btnPlotKlambdas.Visible = false;
                btnClose.Visible = false;
            }
            if (Common.ikalam == 1) // If Ka-lambdas are being printed, show table and controls.
            {
                dataGridView2.Visible = true;
                lblKDepth.Visible = true;
                lblKLat.Visible = true;
                lblKSeason.Visible = true;
                lblKSolar.Visible = true;
                cmbKDepth.Visible = true;
                cmbKLat.Visible = true;
                cmbKSeason.Visible = true;
                cmbKSolar.Visible = true;
                btnPlotKlambdas.Visible = true;
                btnClose.Visible = true;
            }

            if (Common.contaminantType == "Biological")
            {
                lblDesclaimer.Visible = true;
            }
            else
            {
                lblDesclaimer.Visible = false;
            }

            // Populate the latitude-selection combo boxes.
            for (int i = 0; i < Common.ilatsw; i++)
            {
                string value = "";

                if (Common.ityp == 0)  // Using typical ephemeride values.
                {
                    value = Common.ilattm[i].ToString();
                }
                else  // Non-typical ephemeride values.
                {
                    value = Common.typlat.ToString();
                }

                cmbLat.Items.Add(value);
                cmbKLat.Items.Add(value);
            }
            cmbLat.SelectedIndex = 0;  // Set a default selection for this combo box.
            cmbKLat.SelectedIndex = 0;  // Set a default selection for this combo box.

            // Populate the season-selection combo boxes.  Check whether using typical 
            // ephemeride values or specific times and locations.
            if (Common.ityp == 0)   // Using typical ephemeride values.
            {
                for (int i = 0; i < Common.iseasw; i++)
                {
                    string value = Common.sease[i];
                    cmbSeasons.Items.Add(value);
                    cmbKSeason.Items.Add(value);
                }
                cmbSeasons.SelectedIndex = 0;

                lblSeasons.Visible = true;
                cmbSeasons.Visible = true;
                if (Common.ikalam == 1)
                {
                    lblKSeason.Visible = true;
                    cmbKSeason.Visible = true;
                }
            }
            if (Common.ityp == 1)   // Non-typical ephemeride values: season selection is not needed.
            {
                lblSeasons.Visible = false;
                cmbSeasons.Visible = false;
                lblKSeason.Visible = false;
                cmbKSeason.Visible = false;
            }

            // Populate the depth-selection combo box of the photolysis rates.
            for (int irow = 0; irow < dataGridView1.Rows.Count; irow++)
            {
                string sdepth = dataGridView1.Rows[irow].Cells["Summary Items"].Value.ToString();
                if (!sdepth.Equals("") && sdepth.Contains("Depth (cm) ="))
                {
                    int startPos = sdepth.IndexOf("=") + 2;
                    int length = sdepth.Length - startPos;
                    string sub = sdepth.Substring(startPos, length);
                    if (!cmbDepth.Items.Contains(sub)) cmbDepth.Items.Add(sub);
                }
            }
            cmbDepth.SelectedIndex = 0;

            // Populate the depth-selection combo box of the Ka-Lambdas.
            for (int irow = 0; irow < dataGridView2.Rows.Count; irow++)
            {
                string sdepth = dataGridView2.Rows[irow].Cells["Depth (cm)"].Value.ToString();
                if (!sdepth.Equals(""))
                {
                    if (!cmbKDepth.Items.Contains(sdepth)) cmbKDepth.Items.Add(sdepth);
                }
            }
            cmbKDepth.SelectedIndex = 0;

            if (Common.ityp == 0) cmbKSeason.SelectedIndex = 0;

            // Populate the solar altitude selection combo box for the case of Ka-lambdas.
            if (Common.ikalam == 1)
            {
                string itemLat = cmbKLat.SelectedItem.ToString();
                string itemSeason = "";
                if (Common.ityp == 0) itemSeason = cmbKSeason.SelectedItem.ToString();
                string itemDepth = cmbKDepth.SelectedItem.ToString();

                for (int irow = 0; irow < dataGridView2.Rows.Count; irow++)
                {
                    if (Common.ityp == 0)
                    {
                        string vLat = dataGridView2.Rows[irow].Cells["Latitude"].Value.ToString();
                        string vSea = dataGridView2.Rows[irow].Cells["Season"].Value.ToString();
                        if (vLat.Equals(itemLat) && vSea.Equals(itemSeason.ToUpper()))
                        {
                            string solar = dataGridView2.Rows[irow].Cells["SOLAR ALTITUDE"].Value.ToString();
                            if (!cmbKSolar.Items.Contains(solar)) cmbKSolar.Items.Add(solar);
                        }
                    }
                    else
                    {
                        string vDepth = dataGridView2.Rows[irow].Cells["Depth (cm)"].Value.ToString();
                        if (vDepth.Equals(itemDepth))
                        {
                            string solar = dataGridView2.Rows[irow].Cells["SOLAR ALTITUDE"].Value.ToString();
                            if (!cmbKSolar.Items.Contains(solar)) cmbKSolar.Items.Add(solar);
                        }
                    }
                }
                cmbKSolar.SelectedIndex = 0;   // Set default selection for solar altitude combo box.
            }

            // Populate X and Y axes combo boxes and set default selections.
            cmbX.Items.Add("Time of Day (hours)");
            cmbX.Items.Add("Solar Altitude");
            cmbX.SelectedIndex = 0;

            cmbY.Items.Add("Rate Constant (sec**-1)");
            cmbY.SelectedIndex = 0;
        }


        /// <summary>
        /// This method handles the plotting of photolysis rates as a function of hours.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnPlot_Click(object sender, EventArgs e)
        {
            int initialRow = 0, endingRow = 0;
            bool foundInitialIndex = false;

            DataTable dt = new DataTable();
            DataRow dr1 = null;

            // Get the selected latitude.
            string itemLat = cmbLat.SelectedItem.ToString();

            // Get the selected depth.
            string itemDepth = cmbDepth.SelectedItem.ToString();

            // Get the selected season.
            string itemSeason = "";
            if (Common.ityp == 0) itemSeason = cmbSeasons.SelectedItem.ToString();

            // Get the selected X and Y axes.
            string itemX = cmbX.SelectedItem.ToString();
            string itemY = cmbY.SelectedItem.ToString();

            // Add X and Y axes to table.
            dt.Columns.Add(itemX, typeof(double));
            dt.Columns.Add(itemY, typeof(double));

            // Get the starting and ending indices of the data to be read off the table.
            string valueLat;
            string valueSeason;
            string valueDepth;
            for (int irow = 0; irow < dataGridView1.Rows.Count; irow++)
            {
                valueLat = dataGridView1.Rows[irow].Cells["Latitude"].Value.ToString();
                if (valueLat.Equals("")) continue;

                int j = irow;
                string sdepth = dataGridView1.Rows[j].Cells["Summary Items"].Value.ToString();
                while (!sdepth.Contains("Depth (cm) ="))
                {
                    j++;
                    sdepth = dataGridView1.Rows[j].Cells["Summary Items"].Value.ToString();
                }
                int startPos = sdepth.IndexOf("=") + 2;
                int length = sdepth.Length - startPos;
                string sub = sdepth.Substring(startPos, length);
                valueDepth = sub;

                if (Common.ityp == 0)
                {
                    valueSeason = dataGridView1.Rows[irow].Cells["Season"].Value.ToString();
                    if (foundInitialIndex && ((!valueLat.Equals(itemLat)) || (!valueSeason.Equals(itemSeason.ToUpper())) || (!valueDepth.Equals(itemDepth))))
                    {
                        endingRow = irow - 5;
                        foundInitialIndex = false;
                        break;
                    }
                    if (valueLat.Equals(itemLat) && valueSeason.Equals(itemSeason.ToUpper()) && valueDepth.Equals(itemDepth))
                    {
                        initialRow = irow;
                        foundInitialIndex = true;
                    }
                }

                if (Common.ityp == 1)
                {
                    if (foundInitialIndex && (!valueLat.Equals(itemLat) || !valueDepth.Equals(itemDepth)))
                    {
                        endingRow = irow - 5;
                        foundInitialIndex = false;
                        break;
                    }
                    if (valueLat.Equals(itemLat) && valueDepth.Equals(itemDepth))
                    {
                        initialRow = irow;
                        foundInitialIndex = true;
                    }
                }
            }

            if (foundInitialIndex) endingRow = dataGridView1.Rows.Count - 5;

            for (int i = initialRow; i < endingRow; i++)
            {
                dr1 = dt.NewRow();
                dr1[itemX] = dataGridView1.Rows[i].Cells[itemX].Value.ToString();
                dr1[itemY] = dataGridView1.Rows[i].Cells[itemY].Value.ToString();
                dt.Rows.Add(dr1);
            }

            FormPlotTimeOfDay ptod = new FormPlotTimeOfDay();
            ptod.setTable(dt, itemLat, itemDepth, itemSeason, itemX, itemY);
            ptod.ShowDialog();
        }

        /// <summary>
        /// This methods handles the plotting of Ka-Lambdas.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnPlotKlambdas_Click(object sender, EventArgs e)
        {
            int initialIndex = 0, endingIndex = 0;
            bool foundInitialIndex = false;

            DataTable dt = new DataTable();
            DataRow dr1 = null;

            // Get selected latitude.
            string itemLat = cmbKLat.SelectedItem.ToString();

            // Get selected depth.
            string itemDepth = cmbKDepth.SelectedItem.ToString();

            // Get selected solar altitude.
            string itemSolar = cmbKSolar.SelectedItem.ToString();

            // Get the selected season if using typical ephemeride values.
            string itemSeason = "";
            if (Common.ityp == 0) itemSeason = cmbKSeason.SelectedItem.ToString();

            // Add columns to the table that will hold the data to be plotted.
            dt.Columns.Add("Wavelength (nm)");
            dt.Columns.Add("Ka-Lambda (sec^(-1) nm^(-1))");

            string valueLat;
            string valueSeason;
            string valueDepth;
            string valueSolar;
            for (int irow = 0; irow < dataGridView2.Rows.Count; irow++)
            {
                valueLat = dataGridView2.Rows[irow].Cells["Latitude"].Value.ToString();
                if (valueLat.Equals("")) continue;
                valueDepth = dataGridView2.Rows[irow].Cells["Depth (cm)"].Value.ToString();
                valueSolar = dataGridView2.Rows[irow].Cells["Solar Altitude"].Value.ToString();

                if (Common.ityp == 0)
                {
                    valueSeason = dataGridView2.Rows[irow].Cells["Season"].Value.ToString();
                    if (foundInitialIndex && ((!valueLat.Equals(itemLat)) ||
                        (!valueSeason.Equals(itemSeason.ToUpper())) ||
                        (!valueDepth.Equals(itemDepth)) || (!valueSolar.Equals(itemSolar))))
                    {
                        endingIndex = irow - 1;
                        foundInitialIndex = false;
                        break;
                    }
                    if (valueLat.Equals(itemLat) && valueSeason.Equals(itemSeason.ToUpper()) &&
                        valueDepth.Equals(itemDepth) && valueSolar.Equals(itemSolar))
                    {
                        initialIndex = irow;
                        foundInitialIndex = true;
                    }
                }

                if (Common.ityp == 1)
                {
                    if (foundInitialIndex && (!valueLat.Equals(itemLat) ||
                        !valueDepth.Equals(itemDepth) || !valueSolar.Equals(itemSolar)))
                    {
                        endingIndex = irow - 1;
                        foundInitialIndex = false;
                        break;
                    }
                    if (valueLat.Equals(itemLat) && valueDepth.Equals(itemDepth) &&
                        valueSolar.Equals(itemSolar))
                    {
                        initialIndex = irow;
                        foundInitialIndex = true;
                    }
                }
            }

            if (foundInitialIndex) endingIndex = dataGridView2.Rows.Count - 1;

            for (int i = initialIndex; i <= endingIndex; i++)
            {
                dr1 = dt.NewRow();
                dr1["Ka-Lambda (sec^(-1) nm^(-1))"] = dataGridView2.Rows[i].Cells["Ka-Lambda (sec^(-1) nm^(-1))"].Value.ToString();
                dr1["Wavelength (nm)"] = dataGridView2.Rows[i].Cells["Wavelength (nm)"].Value.ToString();
                dt.Rows.Add(dr1);
            }

            FormPlotKlambdas klam = new FormPlotKlambdas();
            klam.setTable(dt, itemLat, itemDepth, itemSeason, itemSolar);
            klam.ShowDialog();
        }

        /// <summary>
        /// This method checks whether the index of the latitude-selection combo box has been 
        /// changed, and if so it proceeds to populate the solar altitude combo box with the 
        /// solar altitude range that corresponds to the selected season and latitude.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmbKLat_SelectedIndexChanged(object sender, EventArgs e)
        {
            cmbKSolar.Items.Clear();
            string itemLat = cmbKLat.SelectedItem.ToString();
            string itemSeason = "";
            if (Common.ityp == 0)
            {
                if (cmbKSeason.SelectedIndex < 0) return;  // Need to have a season selected first.
                itemSeason = cmbKSeason.SelectedItem.ToString();
            }

            for (int irow = 0; irow < dataGridView2.Rows.Count; irow++)
            {
                if (Common.ityp == 0)
                {
                    string vLat = dataGridView2.Rows[irow].Cells["Latitude"].Value.ToString();
                    string vSea = dataGridView2.Rows[irow].Cells["Season"].Value.ToString();
                    if (vLat.Equals(itemLat) && vSea.Equals(itemSeason.ToUpper()))
                    {
                        string solar = dataGridView2.Rows[irow].Cells["Solar Altitude"].Value.ToString();
                        if (!cmbKSolar.Items.Contains(solar)) cmbKSolar.Items.Add(solar);
                    }
                }
                else  // Common.ityp == 1
                {
                    string vLat = dataGridView2.Rows[irow].Cells["Latitude"].Value.ToString();
                    if (vLat.Equals(itemLat))
                    {
                        string solar = dataGridView2.Rows[irow].Cells["Solar Altitude"].Value.ToString();
                        if (!cmbKSolar.Items.Contains(solar)) cmbKSolar.Items.Add(solar);
                    }
                }
            }
            cmbKSolar.SelectedIndex = 0;   // Set default selection for solar altitude combo box.
        }

        /// <summary>
        /// This method checks whether the index of the seasons-selection combo box has been 
        /// changed, and if so it proceeds to populate the solar altitude combo box with the 
        /// solar altitude range that corresponds to the selected season and latitude.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmbKSeason_SelectedIndexChanged(object sender, EventArgs e)
        {
            cmbKSolar.Items.Clear();
            string itemLat = cmbKLat.SelectedItem.ToString();
            string itemSeason = cmbKSeason.SelectedItem.ToString();

            for (int irow = 0; irow < dataGridView2.Rows.Count; irow++)
            {
                string vLat = dataGridView2.Rows[irow].Cells["Latitude"].Value.ToString();
                string vSea = dataGridView2.Rows[irow].Cells["Season"].Value.ToString();
                if (vLat.Equals(itemLat) && vSea.Equals(itemSeason.ToUpper()))
                {
                    string solar = dataGridView2.Rows[irow].Cells["Solar Altitude"].Value.ToString();
                    if (!cmbKSolar.Items.Contains(solar)) cmbKSolar.Items.Add(solar);
                }
            }
            cmbKSolar.SelectedIndex = 0;   // Set default selection for solar altitude combo box.
        }

        private void btnCloseForm_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void TodForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = false;
        }

        private void btnMultiPlot_Click(object sender, EventArgs e)
        {
            int numItems;
            string itemLat, itemX, itemY;
            string itemSeason = "";
            string strLegend = "";
            DataTable[] dt;

            if (radioButton1.Checked == true)  // Fixed latitude and season vs depths.
            {
                strLegend = "Depth = ";
                numItems = cmbDepth.Items.Count;
                dt = new DataTable[numItems];
                String[] valDepth = new String[numItems];

                // Get the selected latitude.
                itemLat = cmbLat.SelectedItem.ToString();

                // Get the selected season.
                itemSeason = "";
                if (Common.ityp == 0) itemSeason = cmbSeasons.SelectedItem.ToString();

                // Get the selected X and Y axes.
                itemX = cmbX.SelectedItem.ToString();
                itemY = cmbY.SelectedItem.ToString();

                for (int i = 0; i < numItems; i++)
                {
                    int initialRow = 0, endingRow = 0;
                    bool foundInitialIndex = false;
                    DataRow dr1 = null;

                    dt[i] = new DataTable();
                    valDepth[i] = cmbDepth.Items[i].ToString();

                    // Get the selected depth.
                    string itemDepth = valDepth[i];

                    // Add X and Y axes to table.
                    dt[i].Columns.Add(itemX, typeof(double));
                    dt[i].Columns.Add(itemY, typeof(double));

                    // Get the starting and ending indices of the data to be read off the table.
                    string valueLat;
                    string valueSeason;
                    string valueDepth;
                    for (int irow = 0; irow < dataGridView1.Rows.Count; irow++)
                    {
                        valueLat = dataGridView1.Rows[irow].Cells["Latitude"].Value.ToString();
                        if (valueLat.Equals("")) continue;

                        int j = irow;
                        string sdepth = dataGridView1.Rows[j].Cells["Summary Items"].Value.ToString();
                        while (!sdepth.Contains("Depth (cm) ="))
                        {
                            j++;
                            sdepth = dataGridView1.Rows[j].Cells["Summary Items"].Value.ToString();
                        }
                        int startPos = sdepth.IndexOf("=") + 2;
                        int length = sdepth.Length - startPos;
                        string sub = sdepth.Substring(startPos, length);
                        valueDepth = sub;

                        if (Common.ityp == 0)
                        {
                            valueSeason = dataGridView1.Rows[irow].Cells["Season"].Value.ToString();
                            if (foundInitialIndex && ((!valueLat.Equals(itemLat)) || (!valueSeason.Equals(itemSeason.ToUpper())) || (!valueDepth.Equals(itemDepth))))
                            {
                                endingRow = irow - 5;
                                foundInitialIndex = false;
                                break;
                            }
                            if (valueLat.Equals(itemLat) && valueSeason.Equals(itemSeason.ToUpper()) && valueDepth.Equals(itemDepth))
                            {
                                initialRow = irow;
                                foundInitialIndex = true;
                            }
                        }

                        if (Common.ityp == 1)
                        {
                            if (foundInitialIndex && (!valueLat.Equals(itemLat) || !valueDepth.Equals(itemDepth)))
                            {
                                endingRow = irow - 5;
                                foundInitialIndex = false;
                                break;
                            }
                            if (valueLat.Equals(itemLat) && valueDepth.Equals(itemDepth))
                            {
                                initialRow = irow;
                                foundInitialIndex = true;
                            }
                        }
                    }

                    if (foundInitialIndex) endingRow = dataGridView1.Rows.Count - 5;

                    for (int j = initialRow; j < endingRow; j++)
                    {
                        dr1 = dt[i].NewRow();
                        dr1[itemX] = dataGridView1.Rows[j].Cells[itemX].Value.ToString();
                        dr1[itemY] = dataGridView1.Rows[j].Cells[itemY].Value.ToString();
                        dt[i].Rows.Add(dr1);
                    }

                }
                FormMultiplePlotsTod mtod = new FormMultiplePlotsTod();
                mtod.setTable(dt, valDepth, strLegend, " cm", itemLat, itemSeason, itemX, itemY);
                mtod.ShowDialog();
            }

            if (radioButton2.Checked == true)  // Fixed depth and season vs latitudes.
            {
                 strLegend = "Latitude = ";
                 numItems = cmbLat.Items.Count;
                 dt = new DataTable[numItems];
                 String[] valLat = new String[numItems];

                    // Get the selected depth.
                    string itemDepth = cmbDepth.SelectedItem.ToString();

                    // Get the selected season.
                    itemSeason = "";
                    if (Common.ityp == 0) itemSeason = cmbSeasons.SelectedItem.ToString();

                    // Get the selected X and Y axes.
                    itemX = cmbX.SelectedItem.ToString();
                    itemY = cmbY.SelectedItem.ToString();

                    for (int i = 0; i < numItems; i++)
                    {
                        int initialRow = 0, endingRow = 0;
                        bool foundInitialIndex = false;
                        DataRow dr1 = null;

                        dt[i] = new DataTable();
                        valLat[i] = cmbLat.Items[i].ToString();

                        // Get the selected latitude.
                        itemLat = valLat[i];

                        // Add X and Y axes to table.
                        dt[i].Columns.Add(itemX, typeof(double));
                        dt[i].Columns.Add(itemY, typeof(double));

                        // Get the starting and ending indices of the data to be read off the table.
                        string valueLat;
                        string valueSeason;
                        string valueDepth;
                        for (int irow = 0; irow < dataGridView1.Rows.Count; irow++)
                        {
                            valueLat = dataGridView1.Rows[irow].Cells["Latitude"].Value.ToString();
                            if (valueLat.Equals("")) continue;

                            int j = irow;
                            string sdepth = dataGridView1.Rows[j].Cells["Summary Items"].Value.ToString();
                            while (!sdepth.Contains("Depth (cm) ="))
                            {
                                j++;
                                sdepth = dataGridView1.Rows[j].Cells["Summary Items"].Value.ToString();
                            }
                            int startPos = sdepth.IndexOf("=") + 2;
                            int length = sdepth.Length - startPos;
                            string sub = sdepth.Substring(startPos, length);
                            valueDepth = sub;

                            if (Common.ityp == 0)
                            {
                                valueSeason = dataGridView1.Rows[irow].Cells["Season"].Value.ToString();
                                if (foundInitialIndex && ((!valueLat.Equals(itemLat)) || (!valueSeason.Equals(itemSeason.ToUpper())) || (!valueDepth.Equals(itemDepth))))
                                {
                                    endingRow = irow - 5;
                                    foundInitialIndex = false;
                                    break;
                                }
                                if (valueLat.Equals(itemLat) && valueSeason.Equals(itemSeason.ToUpper()) && valueDepth.Equals(itemDepth))
                                {
                                    initialRow = irow;
                                    foundInitialIndex = true;
                                }
                            }

                            if (Common.ityp == 1)
                            {
                                if (foundInitialIndex && (!valueLat.Equals(itemLat) || !valueDepth.Equals(itemDepth)))
                                {
                                    endingRow = irow - 5;
                                    foundInitialIndex = false;
                                    break;
                                }
                                if (valueLat.Equals(itemLat) && valueDepth.Equals(itemDepth))
                                {
                                    initialRow = irow;
                                    foundInitialIndex = true;
                                }
                            }
                        }

                        if (foundInitialIndex) endingRow = dataGridView1.Rows.Count - 5;

                        for (int j = initialRow; j < endingRow; j++)
                        {
                            dr1 = dt[i].NewRow();
                            dr1[itemX] = dataGridView1.Rows[j].Cells[itemX].Value.ToString();
                            dr1[itemY] = dataGridView1.Rows[j].Cells[itemY].Value.ToString();
                            dt[i].Rows.Add(dr1);
                        }
                    }
                    FormMultiplePlotsTod mtod = new FormMultiplePlotsTod();
                    mtod.setTable(dt, valLat, strLegend, " degrees north", itemDepth, itemSeason, itemX, itemY);
                    mtod.ShowDialog();
            }

                    
        }
            
    }
}
