﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO;
using System.Data;

namespace GCSOLAR
{
    class Utility
    {
        public static string errorMessage = "";
        const double minMinwave = 280.0;
        const double maxMinwave = 297.5;
        const double minMaxwave = 300.0;
        const double maxMaxwave = 800.0;
        const int minNumLats = 1;
        const int maxNumLats = 10;
        const double minLongitude = 0.0;
        const double maxLongitude = 180.0;
        const double minInitialDepth = 0.0;
        const double maxInitialDepth = 0.01;
        const double minFinalDepth = 1.0;
        const double maxFinalDepth = 1000.0;
        const double minDepthIncr = 0.001;
        const double maxDepthIncr = 20.0;
        const double minDeltaz = 0.001;
        const double maxDeltaz = 0.1;
        const double minOzone = 0.0001;
        const double maxOzone = 1.0;
        const double minQY = 0.1;
        const double maxQY = 1.0;
        const double minElevation = 0.0;
        const double maxElevation = 10000.0;
        const double minRefractive = 0.1;
        const double maxRefractive = 2.0;


        public static bool IsValidData(TextBox txtMinwave, string nameMinwave,
                                       TextBox txtMaxwave, string nameMaxwave,
                                       TextBox txtNumLats, string nameNumLats, 
                                       TextBox txtLongitude, string nameLongitude,
                                       TextBox txtInitialDepth, string nameInitialDepth, 
                                       TextBox txtFinalDepth, string nameFinalDepth,
                                       TextBox txtDepthIncr, string nameDepthIncr,
                                       TextBox txtDeltaz, string nameDeltaz,
                                       TextBox txtOzone, string nameOzone,
                                       TextBox txtQY, string nameQY,
                                       TextBox txtElevation, string nameElevation,
                                       TextBox txtRefractive, string nameRefractive)
        {
            bool result;
            result = IsPresent(txtMinwave, nameMinwave) && IsDouble(txtMinwave, nameMinwave) && IsWithinRange(txtMinwave, nameMinwave, minMinwave, maxMinwave) &&
                     IsPresent(txtMaxwave, nameMaxwave) && IsDouble(txtMaxwave, nameMaxwave) && IsWithinRange(txtMaxwave, nameMaxwave, minMaxwave, maxMaxwave) &&
                     IsPresent(txtNumLats, nameNumLats) && IsInteger(txtNumLats, nameNumLats) && IsIntWithinRange(txtNumLats, nameNumLats, minNumLats, maxNumLats) &&
                     IsPresent(txtLongitude, nameLongitude) && IsDouble(txtLongitude, nameLongitude) && IsWithinRange(txtLongitude, nameLongitude, minLongitude, maxLongitude) &&
                     IsPresent(txtInitialDepth, nameInitialDepth) && IsDouble(txtInitialDepth, nameInitialDepth) && IsWithinRange(txtInitialDepth, nameInitialDepth, minInitialDepth, maxInitialDepth) &&
                     IsPresent(txtFinalDepth, nameFinalDepth) && IsDouble(txtFinalDepth, nameFinalDepth) && IsWithinRange(txtFinalDepth, nameFinalDepth, minFinalDepth, maxFinalDepth) &&
                     IsPresent(txtDepthIncr, nameDepthIncr) && IsDouble(txtDepthIncr, nameDepthIncr) && IsWithinRange(txtDepthIncr, nameDepthIncr, minDepthIncr, maxDepthIncr) &&
                     IsPresent(txtOzone, nameOzone) && IsDouble(txtOzone, nameOzone) && IsWithinRange(txtOzone, nameOzone, minOzone, maxOzone) &&
                     IsPresent(txtQY, nameQY) && IsDouble(txtQY, nameQY) && IsWithinRange(txtQY, nameQY, minQY, maxQY) &&
                     IsPresent(txtElevation, nameElevation) && IsDouble(txtElevation, nameElevation) && IsWithinRange(txtElevation, nameElevation, minElevation, maxElevation) &&
                     IsPresent(txtRefractive, nameRefractive) && IsDouble(txtRefractive, nameRefractive) && IsWithinRange(txtRefractive, nameRefractive, minRefractive, maxRefractive);
                     
            return result;
        }

        public static bool IsPresent(TextBox textBox, string name)
        {
            if (textBox.Text == "")
            {
                errorMessage = name + " is a required field.";
                textBox.Focus();
                return false;
            }
            return true;
        }

        public static bool IsDouble(TextBox textBox, string name)
        {
            double number = 0;
            if (double.TryParse(textBox.Text, out number))
            {
                return true;
            }
            else
            {
                errorMessage = name + " must be a double value.";
                textBox.Focus();
                return false;
            }
        }

        public static bool IsInteger(TextBox textBox, string name)
        {
            int number = 0;
            if (int.TryParse(textBox.Text, out number))
            {
                return true;
            }
            else
            {
                errorMessage = name + " must be an integer value.";
                textBox.Focus();
                return false;
            }
        }

        public static bool IsWithinRange(TextBox textBox, string name, double min, double max)
        {
            double number = Convert.ToDouble(textBox.Text);
            if (number < min || number > max)
            {
                errorMessage = name + " must be a number from " + min + " to " + max + ".";
                textBox.Focus();
                return false;
            }
            return true;
        }

        public static bool IsIntWithinRange(TextBox textBox, string name, int min, int max)
        {
            int number = Convert.ToInt32(textBox.Text);
            if (number < min || number > max)
            {
                errorMessage = name + " must be a number from " + min + " to " + max + ".";
                textBox.Focus();
                return false;
            }
            return true;
        }

        public static void ExportTextBasedFile(DataGridView dataGridView1)
        {
            // Create an instance of the save file dialog box.
            SaveFileDialog saveFileDialog1 = new SaveFileDialog();

            // Set filter options and filter index.
            saveFileDialog1.Filter = "CSV Files (.csv)|*.csv|All Files (*.*)|*.*";
            saveFileDialog1.FilterIndex = 1;

            saveFileDialog1.RestoreDirectory = true;

            // Call the ShowDialog method to show the dialog box.
            System.Windows.Forms.DialogResult userClickedOK = saveFileDialog1.ShowDialog();

            // Process input if the user clicked OK.
            if (userClickedOK == System.Windows.Forms.DialogResult.OK)
            {
                string name = saveFileDialog1.FileName;

                if (dataGridView1.Rows.Count > 0)
                {
                    try
                    {
                        // Bind Grid Data to Datatable
                        DataTable dt = new DataTable();
                        foreach (DataGridViewColumn col in dataGridView1.Columns)
                        {
                            dt.Columns.Add(col.HeaderText, col.ValueType);
                        }
                        int count = 0;
                        foreach (DataGridViewRow row in dataGridView1.Rows)
                        {
                            if (count <= dataGridView1.Rows.Count - 1)
                            {
                                dt.Rows.Add();
                                foreach (DataGridViewCell cell in row.Cells)
                                {
                                    dt.Rows[dt.Rows.Count - 1][cell.ColumnIndex] = cell.Value.ToString();
                                }
                            }
                            count++;
                        }

                        // Bind table data to Stream Writer to export data to respective folder
                        string destinationPath = name;
                        StreamWriter wr = new StreamWriter(destinationPath);

                        // Write Columns to excel file
                        for (int i = 0; i < dt.Columns.Count; i++)
                        {
                            if (i != dt.Columns.Count - 1)
                            {
                                wr.Write(dt.Columns[i].ToString().ToUpper() + ",");
                            }
                            else
                            {
                                wr.Write(dt.Columns[i].ToString().ToUpper());
                            }
                            
                        }
                        wr.WriteLine();

                        //write rows to excel file
                        for (int i = 0; i < (dt.Rows.Count); i++)
                        {
                            for (int j = 0; j < dt.Columns.Count; j++)
                            {
                                if (dt.Rows[i][j] != null)
                                {
                                    if (j != dt.Columns.Count - 1)
                                    {
                                        wr.Write(Convert.ToString(dt.Rows[i][j]) + ",");
                                    }
                                    else
                                    {
                                        wr.Write(Convert.ToString(dt.Rows[i][j]));
                                    }                                  
                                }
                                else
                                {
                                    wr.Write(",");
                                }
                            }
                            wr.WriteLine();
                        }
                        wr.Close();
                        MessageBox.Show("Data was successfully saved.");
                    }
                    catch (Exception ex)
                    {
                        throw ex;
                    }
                }
            }
        }

        public static void ExportColumnDataFile(DataGridView dataGridView1, int colIndex)
        {
            // Create an instance of the save file dialog box.
            SaveFileDialog saveFileDialog1 = new SaveFileDialog();

            // Set filter options and filter index.
            saveFileDialog1.Filter = "CSV Files (.csv)|*.csv|All Files (*.*)|*.*";
            saveFileDialog1.FilterIndex = 1;

            saveFileDialog1.RestoreDirectory = true;

            // Call the ShowDialog method to show the dialog box.
            System.Windows.Forms.DialogResult userClickedOK = saveFileDialog1.ShowDialog();

            // Process input if the user clicked OK.
            if (userClickedOK == System.Windows.Forms.DialogResult.OK)
            {
                string name = saveFileDialog1.FileName;

                if (dataGridView1.Rows.Count > 0)
                {
                    try
                    {
                        // Bind Grid Data to Datatable
                        DataTable dt = new DataTable();
                        foreach (DataGridViewColumn col in dataGridView1.Columns)
                        {
                            if (col.Index != colIndex)
                            {
                                dt.Columns.Add(col.HeaderText, col.ValueType);
                            }
                            
                        }
                        int count = 0;
                        foreach (DataGridViewRow row in dataGridView1.Rows)
                        {
                            if (count <= dataGridView1.Rows.Count - 1)
                            {
                                dt.Rows.Add();
                                int colCounter = 0;
                                foreach (DataGridViewCell cell in row.Cells)
                                {
                                    if (cell.ColumnIndex != colIndex)
                                    {
                                        dt.Rows[dt.Rows.Count - 1][colCounter] = cell.Value.ToString();
                                        colCounter++;
                                    }                  
                                }
                            }
                            count++;
                        }

                        // Bind table data to Stream Writer to export data to respective folder
                        string destinationPath = name;
                        StreamWriter wr = new StreamWriter(destinationPath);

                        // Write Columns to excel file
                        for (int i = 0; i < dt.Columns.Count; i++)
                        {
                            if (i != dt.Columns.Count - 1)
                            {
                                wr.Write(dt.Columns[i].ToString().ToUpper() + ",");
                            }
                            else
                            {
                                wr.Write(dt.Columns[i].ToString().ToUpper());
                            }

                        }
                        wr.WriteLine();

                        //write rows to excel file
                        for (int i = 0; i < (dt.Rows.Count); i++)
                        {
                            for (int j = 0; j < dt.Columns.Count; j++)
                            {
                                if (dt.Rows[i][j] != null)
                                {
                                    if (j != dt.Columns.Count - 1)
                                    {
                                        wr.Write(Convert.ToString(dt.Rows[i][j]) + ",");
                                    }
                                    else
                                    {
                                        wr.Write(Convert.ToString(dt.Rows[i][j]));
                                    }
                                }
                                else
                                {
                                    wr.Write(",");
                                }
                            }
                            wr.WriteLine();
                        }
                        wr.Close();
                        MessageBox.Show("Data was successfully saved.");
                    }
                    catch (Exception ex)
                    {
                        throw ex;
                    }
                }
            }
        }


    }
}
